#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in vec3 out_normal;
in vec3 FragPos;
in vec2 UV;

layout(location=6)uniform vec3 viewPos;

layout(location=7)uniform vec4 constants;
layout(location=8)uniform sampler2D text;

layout(location=9)uniform int numLights;
layout(location=10)uniform struct Light {
	vec4 position;
	vec3 intensities;
	vec3 attenuation;
	float coneAngleIn;
	float coneAngleOut;
	vec3 coneDirection;
} lights[10];
//LOCATION 10-15 in steps size 5!

void main() {

	vec3 result = vec3(0);
	vec3 Ia = vec3(1);
	float factor = 1.0f;
	
	//ambient
	vec3 ambientResult = constants.x * Ia;
	
	vec3 diffuseResult = vec3(0);
	vec3 specularResult = vec3(0);
	
	for (int i = 0; i < numLights; ++i){
		float F = 1.0f;
		vec3 l = vec3(0);
		if (lights[i].position.w <= 0.5) {//directional light
			l = normalize(vec3(-lights[i].position)); 
		} else if (lights[i].position.w > 0.5 && lights[i].position.w < 1.5) {//point light
			vec3 h = (lights[i].position.xyz - FragPos);
			l = normalize(h);
			float d = length(h);
			F = 1.0f / (lights[i].attenuation.x + lights[i].attenuation.y*d + lights[i].attenuation.z*d*d);
		} else {//spot light
			l = normalize(lights[i].position.xyz - FragPos);
			float surfaceLightAngle = acos(dot(-l,normalize(lights[i].coneDirection)));
			if (surfaceLightAngle > lights[i].coneAngleOut){
				F = 0.0f;
			} else if (surfaceLightAngle < lights[i].coneAngleIn) {
				vec3 h = (lights[i].position.xyz - FragPos);
				float d = length(h);
				F = 1.0f/(lights[i].attenuation.x + lights[i].attenuation.y*d + lights[i].attenuation.z*d*d);	
			} else {
				factor = smoothstep(lights[i].coneAngleOut, lights[i].coneAngleIn , surfaceLightAngle);
				vec3 h = (lights[i].position.xyz - FragPos);
				float d = length(h);
				F = 1.0f/(lights[i].attenuation.x + lights[i].attenuation.y*d + lights[i].attenuation.z*d*d);
				F = F * factor;
			}
		}
		//diffuse
		vec3 n = normalize(out_normal);
		vec3 diffuse = min(max(dot(n, l), 0.0),1.0) * constants.y * F * lights[i].intensities;
		
		//specular
		float spec = 0.0f;
		vec3 rd = reflect(-l,n);
		vec3 v = normalize(viewPos-FragPos);
		if (dot(n, l) > 0.0) {
			spec = pow(min(max(dot(v,rd),0.0),1.0),constants.w);
		}
		vec3 specular = spec * constants.z * F * lights[i].intensities;
		
		diffuseResult += diffuse;
		specularResult += specular;
	}
	result = texture(text,UV).rgb*(ambientResult + diffuseResult) + specularResult;
	gl_FragColor = vec4(result,0.75f);
}



